import os
import uuid
from abc import abstractmethod


class BaseGetter(object):

    def __init__(self, url):
        self.url = url
        self.temp_dir = None

    def get(self):
        current_directory = os.getcwd()
        final_directory = os.path.join(current_directory, str((uuid.uuid4()))[:8]+"_checks")
        if not os.path.exists(final_directory):
            os.makedirs(final_directory)
        self.temp_dir = final_directory
        return self.do_get()

    @abstractmethod
    def do_get(self):
        raise NotImplementedError()



import json
import logging
from abc import abstractmethod
from cloudsplaining.scan.policy_document import PolicyDocument

from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.multi_signature import multi_signature
from checkov.terraform.checks.utils.iam_cloudformation_document_to_policy_converter import \
    convert_cloudformation_conf_to_iam_policy


class BaseCloudsplainingIAMCheck(BaseResourceCheck):
    def __init__(self, name, id):
        super().__init__(name=name, id=id, categories=CheckCategories.IAM, supported_resources=["AWS::IAM::Policy", "AWS::IAM::ManagedPolicy"])

    def scan_resource_conf(self, conf):
        if 'Properties' in conf.keys():
            props_conf = conf['Properties']
            key = 'PolicyDocument'
            if key in props_conf.keys():
                converted_conf = props_conf[key]
                try:
                    converted_conf = convert_cloudformation_conf_to_iam_policy(converted_conf)
                    key = 'Statement'
                    if key in converted_conf:
                        policy = PolicyDocument(converted_conf)
                        violations = self.cloudsplaining_analysis(policy)
                        if violations:
                            logging.debug("detailed cloudsplainging finding: {}",json.dumps(violations))
                            return CheckResult.FAILED
                except Exception as e:
                    # this might occur with templated iam policies where ARN is not in place or similar
                    logging.debug("could not run cloudsplaining analysis on policy {}", conf)
                    return CheckResult.UNKNOWN
        return CheckResult.PASSED

    @multi_signature()
    @abstractmethod
    def cloudsplaining_analysis(self, policy):
        raise NotImplementedError()



import json
import logging

import yaml

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class SSMSessionManagerDocumentEncryption(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Session Manager data is encrypted in transit"
        id = "CKV_AWS_112"
        supported_resources = ["aws_ssm_document"]
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        if conf.get("document_type") == ["Session"]:
            doc_format = conf.get("document_format", ["JSON"])

            if "content" in conf.keys():
                content = conf["content"][0]
                inputs = None

                if doc_format == ["JSON"] and is_json(content):
                    inputs = json.loads(content).get("inputs", {})
                elif doc_format == ["YAML"] and is_yaml(content):
                    inputs = yaml.safe_load(content).get("inputs", {})

                if inputs and not inputs.get("kmsKeyId"):
                    return CheckResult.FAILED

                return CheckResult.PASSED

        return CheckResult.UNKNOWN


def is_json(data):
    try:
        json.loads(data)
    except ValueError:
        logging.debug(f"could not parse json data: {data}")
        return False
    return True


def is_yaml(data):
    try:
        yaml.safe_load(data)
    except yaml.YAMLError:
        logging.debug(f"could not parse yaml data: {data}")
        return False
    return True


check = SSMSessionManagerDocumentEncryption()



from checkov.terraform.checks.data.BaseCloudsplainingIAMCheck import BaseCloudsplainingIAMCheck


class CloudSplainingPrivilegeEscalation(BaseCloudsplainingIAMCheck):

    def __init__(self):
        name = "Ensure IAM policies does not allow privilege escalation"
        id = "CKV_AWS_110"
        super().__init__(name=name, id=id)

    def cloudsplaining_analysis(self, policy):
        escalation = policy.allows_privilege_escalation
        return escalation


check = CloudSplainingPrivilegeEscalation()
